# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals


__author__ = 'Serkan Hosca'
__author_email__ = 'serkan@hosca.com'
__version__ = '0.0.2'
__description__ = 'SQLAlchemy specific things for django-rest-framework'
